package org.processmining.dataawarereplayer.precision.measure;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Set;

import org.processmining.dataawarereplayer.precision.PrecisionConfig;
import org.processmining.dataawarereplayer.precision.PrecisionMeasureException;
import org.processmining.dataawarereplayer.precision.projection.ProcessProjector;
import org.processmining.dataawarereplayer.precision.projection.ProjectedEvent;
import org.processmining.dataawarereplayer.precision.state.PrecisionState;
import org.processmining.dataawarereplayer.precision.state.StateFactory;
import org.processmining.dataawarereplayer.precision.state.StateTransition;
import org.processmining.dataawarereplayer.precision.state.StateTransitionActivityImpl;
import org.processmining.datapetrinets.DataPetriNet;
import org.processmining.datapetrinets.expression.GuardExpression;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.PNWDTransition;
import org.processmining.models.semantics.petrinet.EfficientPetrinetSemantics;
import org.processmining.plugins.balancedconformance.dataflow.datastate.DataState;
import org.processmining.plugins.balancedconformance.dataflow.util.ExpressionEvaluator;

public final class ActivityPrecisionMeasureImpl extends AbstractDPNPrecisionMeasure {

	private static final String UNASSIGNED_VARIABLES = "Enablement of transition %s cannot be checked because guard %s is referring to unassigned variables. Current assignment %s.";

	private final static class TransitionContinuationImpl implements ProjectedEvent {

		private final Transition nextTransition;

		private TransitionContinuationImpl(Transition nextTransition) {
			this.nextTransition = nextTransition;
		}

		public Set<String> getAttributesWritten() {
			return Collections.emptySet();
		}

		public Object getAttributeValue(String varName) {
			return null;
		}

		public Transition getTransition() {
			return nextTransition;
		}

		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result + ((nextTransition == null) ? 0 : nextTransition.hashCode());
			return result;
		}

		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (!(obj instanceof TransitionContinuationImpl))
				return false;
			TransitionContinuationImpl other = (TransitionContinuationImpl) obj;
			if (nextTransition == null) {
				if (other.nextTransition != null)
					return false;
			} else if (!nextTransition.equals(other.nextTransition))
				return false;
			return true;
		}

	}

	public ActivityPrecisionMeasureImpl(PrecisionConfig config, ProcessProjector processProjector,
			StateFactory stateFactory, DataPetriNet net) {
		super(config, processProjector, stateFactory, net);
	}

	protected StateTransition newStateTransition(ProjectedEvent event) {
		return new StateTransitionActivityImpl(event.getTransition());
	}

	protected Collection<StateTransition> recordPossibilities(DataState variableAssignment, PrecisionState state,
			EfficientPetrinetSemantics petriNetSemantics) throws PrecisionMeasureException {
		Collection<StateTransition> continuations = new ArrayList<>();
		for (Transition transition : transitions) {
			if (petriNetSemantics.isEnabled(transition)) {
				if (transition instanceof PNWDTransition
						&& ((PNWDTransition) transition).getGuardExpression() != null) {
					GuardExpression guardExpression = ((PNWDTransition) transition).getGuardExpression();
					boolean allVariablesAssigned = ExpressionEvaluator.checkAllVariableNotNull(guardExpression,
							variableAssignment);
					if (allVariablesAssigned) {
						if (ExpressionEvaluator.isTrue(guardExpression, variableAssignment,
								processProjector.getAttributeType(), processProjector.getStringDiscretizer())) {
							continuations.add(newStateTransition(new TransitionContinuationImpl(transition)));
						}
					} else {
						// Guard cannot be checked as there is a missing variable, e.g. not initialized or a prime variable
						// Assume it can be satisfied
						if (config.isAssumeUnassignedVariablesFree()) {
							continuations.add(newStateTransition(new TransitionContinuationImpl(transition)));
						} else {
							throw new PrecisionMeasureException(String.format(UNASSIGNED_VARIABLES, transition,
									guardExpression, variableAssignment));
						}
					}
				} else {
					continuations.add(newStateTransition(new TransitionContinuationImpl(transition)));
				}
			}
		}
		return continuations;
	}

}