package org.processmining.dataawarereplayer.precision.measure;

import org.processmining.dataawarereplayer.precision.PrecisionMeasureException;
import org.processmining.dataawarereplayer.precision.projection.ProjectedTrace;
import org.processmining.dataawarereplayer.precision.state.PrecisionState;
import org.processmining.dataawarereplayer.precision.state.StateTransition;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.semantics.petrinet.Marking;

import com.google.common.collect.SetMultimap;

public interface PrecisionMeasure {
	
	public enum ResultMode {
		OVERALL, PLACE_LEVEL
	}

	void processTrace(ProjectedTrace processProjection) throws PrecisionMeasureException;
	
	void collectResults(ResultMode mode);

	double getPrecision();

	double getPrecision(Place p);

	double getPrecision(Marking marking);

	long getObservedBehavior(Place p);

	long getPossibleBehavior(Place p);

	long getPossibleContinuations(Marking marking);

	long getObservedContinuations(Marking marking);

	long getFrequency(PrecisionState state);

	SetMultimap<PrecisionState, StateTransition> getObservedStateTransitions(Place p);
	
	SetMultimap<PrecisionState, StateTransition> getPossibleStateTransitions(Place p);

	SetMultimap<PrecisionState, StateTransition> getPossibleStateTransitions();

	SetMultimap<PrecisionState, StateTransition> getObservedStateTransitions();

	long getObservedLocalBehavior(Place p);

	long getPossibleLocalBehavior(Place p);

	double getLocalPrecision(Place p);

	Object getModel();
}
