package org.processmining.dataawarereplayer.precision.visualizer;

import java.awt.Color;
import java.awt.Dimension;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import org.processmining.dataawarereplayer.precision.PrecisionResult;
import org.processmining.dataawarereplayer.precision.state.PrecisionState;
import org.processmining.dataawarereplayer.precision.state.StateTransition;
import org.processmining.framework.util.ui.widgets.traceview.ProMTraceList;
import org.processmining.framework.util.ui.widgets.traceview.ProMTraceList.TraceBuilder;
import org.processmining.framework.util.ui.widgets.traceview.ProMTraceView.Event;
import org.processmining.framework.util.ui.widgets.traceview.ProMTraceView.Trace;

import com.google.common.collect.SetMultimap;

public final class PrecisionStateListViewUtils {

	private PrecisionStateListViewUtils() {
	}

	public static Collection<PrecisionTrace> createPrecisionTraces(PrecisionResult precisionResult,
			SetMultimap<PrecisionState, StateTransition> possibleStateTransitions,
			SetMultimap<PrecisionState, StateTransition> observedStateTransitions) {
		Map<PrecisionState, Collection<StateTransition>> precisionStateMap = observedStateTransitions.asMap();
		Collection<PrecisionTrace> precisionTraces = new ArrayList<>(precisionStateMap.size());
		for (Iterator<Entry<PrecisionState, Collection<StateTransition>>> iterator = precisionStateMap.entrySet()
				.iterator(); iterator.hasNext();) {
			Entry<PrecisionState, Collection<StateTransition>> observedEntry = iterator.next();
			PrecisionState state = observedEntry.getKey();
			Collection<StateTransition> observerdCont = observedEntry.getValue();
			Collection<StateTransition> possibleCont = possibleStateTransitions.get(state);
			PrecisionTrace precisionTrace = new PrecisionTrace(state, observerdCont, possibleCont,
					precisionResult.getFrequency(state));
			precisionTraces.add(precisionTrace);
		}
		return precisionTraces;
	}

	public static ProMTraceList<PrecisionTrace> createPrecisionTraceList(Collection<PrecisionTrace> precisionTraces) {
		ProMTraceList<PrecisionTrace> precisionTraceList = new ProMTraceList<>(precisionTraces,
				new TraceBuilder<PrecisionTrace>() {

					public Trace<? extends Event> build(final PrecisionTrace element) {
						return element;
					}
				});

		precisionTraceList.setPreferredSize(new Dimension(600, 400));
		precisionTraceList.setOpaque(true);
		precisionTraceList.setBackground(Color.WHITE);
		precisionTraceList.setForeground(Color.BLACK);
		precisionTraceList.setMaxWedgeWidth(800);
		precisionTraceList.sort(new Comparator<PrecisionTrace>() {

			public int compare(PrecisionTrace o1, PrecisionTrace o2) {
				return Long.compare(o2.getFrequency(), o1.getFrequency());
			}
		});
		return precisionTraceList;
	}

}
