package org.processmining.plugins.DataConformance.GUI;

import java.awt.Color;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import org.processmining.framework.util.LevenshteinDistance;
import org.processmining.framework.util.ui.widgets.ProMComboBox;
import org.processmining.framework.util.ui.widgets.ProMPropertiesPanel;

public class MappingPanel<T,V> extends ProMPropertiesPanel 
{
	Map<T,ProMComboBox> comboBoxes=new HashMap<T,ProMComboBox>();
	private final static float lowDistFact=0.3F;
	private final static float highDistFact=0.7F;
	
	public MappingPanel(Collection<T> coll1,Collection<V> coll2) 
	{
		super("Please provide the mapping for the following:");
		Set<Object> internalSet;
		try
		{
			internalSet=new TreeSet<Object>();
			internalSet.addAll(coll2);
			internalSet.add("");
		}
		catch(ClassCastException err)
		{
			internalSet=new HashSet<Object>();
			internalSet.addAll(coll2);
			internalSet.add("");			
		}
		for(T node : new TreeSet<T>(coll1))
		{
			ProMComboBox component=new ProMComboBox(internalSet);
			component.setMinimumSize(null);
			component.setMaximumSize(null);
			component.setPreferredSize(null);
			Object retValue[]=getBestMatch(node.toString(), internalSet);
			
			if (((Integer)retValue[1]) < highDistFact*node.toString().length())
			{
				component.setSelectedItem(retValue[0]);
				if (((Integer)retValue[1]) > lowDistFact*node.toString().length())
				{
					component.setBackground(Color.YELLOW);
					component.setForeground(Color.BLACK);					
				}
			}
				
			comboBoxes.put(node, component);
			this.addProperty(node.toString(), component);
		}
	}
	
	private Object[] getBestMatch(String string, Set<Object> internalSet) {
		LevenshteinDistance ld=new LevenshteinDistance();
		int minDist=Integer.MAX_VALUE;
		Object minObject=null;
		for(Object internal : internalSet)
		{
			int ldist;
			if(internal.toString().length() == 0)
				ldist=string.length();
			else
				ldist=ld.getLevenshteinDistanceLinearSpace(string, internal.toString());
			
			if (ldist<minDist)
			{
				minDist=ldist;
				minObject=internal;
			}
		}
		return new Object[]{minObject,minDist};
	}

	@SuppressWarnings("unchecked")
	public Map<T,V> getMapping(boolean includeNotMapped)
	{
		Map<T,V> retValue=new HashMap<T,V>();
		for(T node : comboBoxes.keySet())
		{
			Object selection=comboBoxes.get(node).getSelectedItem();
			if (selection.equals(""))
				{if (includeNotMapped) retValue.put(node, null);}
			else
				retValue.put(node, (V) selection);
		}
		return retValue;
	}

}
