package org.processmining.plugins.DataConformance.GUI;

import java.util.Map;
import java.util.Map.Entry;
import java.util.Vector;

import javax.swing.DefaultCellEditor;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JTable;

import org.processmining.plugins.DataConformance.framework.ReplayableActivity;

import com.google.common.collect.Ordering;

public class VariableMatchingCostTable<E> extends JTable //ProMTable
{
	VariableMatchingTableModel model;

	@SuppressWarnings("unchecked")
	private Vector<MatchingActivity<? extends ReplayableActivity, E>> getActivityList(
			Map<? extends ReplayableActivity, E> activityMapping) {
		Vector<MatchingActivity<? extends ReplayableActivity, E>> activities = new Vector<MatchingActivity<? extends ReplayableActivity, E>>();

		Ordering<Entry<? extends ReplayableActivity, E>> activityOrdering = new Ordering<Map.Entry<? extends ReplayableActivity, E>>() {

			public int compare(Map.Entry<? extends ReplayableActivity, E> o1,
					Map.Entry<? extends ReplayableActivity, E> o2) {
				return o1.getKey().getLabel().compareTo(o2.getKey().getLabel());
			}

		};

		for (Entry<? extends ReplayableActivity, E> entry : activityOrdering.sortedCopy(activityMapping.entrySet())) {
			if (entry.getValue() != null) {
				MatchingActivity<? extends ReplayableActivity, E> matchNode = new MatchingActivity<ReplayableActivity, E>(
						entry.getKey(), entry.getValue());
				activities.add(matchNode);
			}
		}

		activities.add(MatchingActivity.ALLFLEXNODES);
		return activities;
	}

	private Vector<MatchingXAttribute> getVariableList(Map<String, String> variableMapping) {
		Vector<MatchingXAttribute> variables = new Vector<MatchingXAttribute>();
		Ordering<Entry<String, String>> variableOrdering = new Ordering<Map.Entry<String, String>>() {

			public int compare(Map.Entry<String, String> o1, Map.Entry<String, String> o2) {
				return o1.getKey().compareTo(o2.getKey());
			}

		};

		for (Entry<String, String> entry : variableOrdering.sortedCopy(variableMapping.entrySet())) {
			if (entry.getValue() != null) {
				variables.add(new MatchingXAttribute(entry.getValue(), entry.getKey()));
			} else {
				variables.add(new MatchingXAttribute("", entry.getKey()));
			}
		}
		
		variables.add(MatchingXAttribute.ALLATTRIBUTES);
		return variables;
	}

	public VariableMatchingCostTable(Map<? extends ReplayableActivity, E> activityMapping,
			Map<String, String> variableMapping) {
		super(new VariableMatchingTableModel());
		model = (VariableMatchingTableModel) this.getModel();
		this.getColumnModel().getColumn(0)
				.setCellEditor(new DefaultCellEditor(new JComboBox(getActivityList(activityMapping))));
		this.getColumnModel().getColumn(1)
				.setCellEditor(new DefaultCellEditor(new JComboBox(getVariableList(variableMapping))));
		this.getColumnModel().getColumn(4).setCellEditor(new DefaultCellEditor(new JCheckBox()));
	}

	public void addRow() {
		if (editingRow > 0)
			return;
		int selectedRow = this.getSelectedRow();
		if (selectedRow < 0)
			selectedRow = 0;
		model.addRow(selectedRow);
		model.fireTableDataChanged();
	}

	public void removeRow() {
		if (editingRow > 0)
			return;
		int[] indexes = this.getSelectedRows();
		model.removeRow(indexes);
		model.fireTableDataChanged();
	}

	public void moveUp() {
		if (editingRow > 0)
			return;
		int selectedRow = this.getSelectedRow();
		model.moveUp(selectedRow);
		model.fireTableDataChanged();
		if (selectedRow > 0)
			this.addRowSelectionInterval(selectedRow - 1, selectedRow - 1);
	}

	public void moveDown() {
		if (editingRow > 0)
			return;
		int selectedRow = this.getSelectedRow();
		model.moveDown(selectedRow);
		model.fireTableDataChanged();
		if (selectedRow < model.getRowCount() - 1)
			this.addRowSelectionInterval(selectedRow + 1, selectedRow + 1);
	}

}
