package org.processmining.plugins.DataConformance.visualization.alignment;

import static com.googlecode.cqengine.query.QueryFactory.all;
import static com.googlecode.cqengine.query.QueryFactory.or;

import java.util.Date;

import org.processmining.xesalignmentextension.XAlignmentExtension.XAlignment;
import org.processmining.xesalignmentextension.XAlignmentExtension.XAlignmentMove;
import org.processmining.xeslite.query.AttributeResolver;
import org.processmining.xeslite.query.XIndex;
import org.processmining.xeslite.query.syntax.ParseException;

import com.google.common.base.Function;
import com.google.common.base.Predicate;
import com.google.common.collect.Iterables;
import com.googlecode.cqengine.attribute.Attribute;
import com.googlecode.cqengine.attribute.MultiValueNullableAttribute;
import com.googlecode.cqengine.attribute.SimpleAttribute;
import com.googlecode.cqengine.query.Query;
import com.googlecode.cqengine.query.QueryFactory;
import com.googlecode.cqengine.query.option.QueryOptions;

public class AlignmentQueryPredicate implements Predicate<XAlignment> {

	private final static class AttributeResolverImplementation implements AttributeResolver<XAlignment> {

		public Class<?> getAttributeType(String attributeName) {
			return Double.class;
		}

		public Attribute<XAlignment, String> stringAttribute(String eventName, String attributeName) {
			throw new UnsupportedOperationException("Error in alignment query, cannot handle String attribute");
		}

		public Attribute<XAlignment, String> stringAttribute(String attributeName) {
			throw new UnsupportedOperationException("Error in alignment query, cannot handle String attribute");
		}

		public Attribute<XAlignment, Double> doubleAttribute(String eventName, String attributeName) {
			return doubleAttribute(attributeName);
		}

		public Attribute<XAlignment, Double> doubleAttribute(String attributeName) {
			return fitness();
		}

		public Attribute<XAlignment, Date> dateAttribute(String eventName, String attributeName) {
			throw new UnsupportedOperationException("Error in alignment query, cannot handle Date attribute");
		}

		public Attribute<XAlignment, Date> dateAttribute(String attributeName) {
			throw new UnsupportedOperationException("Error in alignment query, cannot handle Date attribute");
		}

	}

	private Query<XAlignment> query;

	public AlignmentQueryPredicate(String queryString) throws ParseException {
		if (!queryString.isEmpty() && queryString.charAt(0) == '~') {
			if (queryString.length() > 1) {
				queryString = queryString.substring(1);
				query = or(QueryFactory.matchesRegex(traceName(), queryString),
						QueryFactory.matchesRegex(allMoveNames(), queryString));
			} else {
				throw new ParseException("Missing regular expression afterr '~'");
			}
		} else if (!queryString.isEmpty() && queryString.charAt(0) == '%') {
			if (queryString.length() > 1) {
				queryString = queryString.substring(1);
				query = or(QueryFactory.contains(traceName(), queryString),
						QueryFactory.contains(allMoveNames(), queryString));
			} else {
				throw new ParseException("Missing query after '%'");
			}
		} else if (!queryString.isEmpty() && queryString.startsWith("fitness")) {
			query = XIndex.buildQuery(queryString, new AttributeResolverImplementation(), XAlignment.class);
		} else if (!queryString.isEmpty()) {
			query = or(QueryFactory.equal(traceName(), queryString), QueryFactory.equal(allMoveNames(), queryString));
		} else {
			query = all(XAlignment.class);
		}
	}

	public boolean apply(XAlignment a) {
		return query.matches(a, QueryFactory.noQueryOptions());
	}

	private static Attribute<XAlignment, String> traceName() {
		return new SimpleAttribute<XAlignment, String>() {

			public String getValue(XAlignment a, QueryOptions queryOptions) {
				return a.getName();
			}
		};
	}

	private static Attribute<XAlignment, String> allMoveNames() {
		return new MultiValueNullableAttribute<XAlignment, String>(true) {

			public Iterable<String> getNullableValues(XAlignment a, QueryOptions queryOptions) {
				return Iterables.transform(a, new Function<XAlignmentMove, String>() {

					public String apply(XAlignmentMove m) {
						return (m.getModelMove() != null) ? m.getModelMove() : m.getLogMove();
					}
				});
			}
		};
	}

	private static Attribute<XAlignment, Double> fitness() {
		return new SimpleAttribute<XAlignment, Double>() {

			public Double getValue(XAlignment a, QueryOptions queryOptions) {
				return a.getFitness();
			}
		};
	}

}
