package org.processmining.plugins.DataConformance.visualization.alignment;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.text.DateFormat;
import java.text.NumberFormat;
import java.util.Date;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;

import javax.swing.DefaultCellEditor;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeBoolean;
import org.deckfour.xes.model.XAttributeContainer;
import org.deckfour.xes.model.XAttributeContinuous;
import org.deckfour.xes.model.XAttributeDiscrete;
import org.deckfour.xes.model.XAttributeID;
import org.deckfour.xes.model.XAttributeList;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XAttributeTimestamp;
import org.deckfour.xes.model.XTrace;
import org.processmining.datapetrinets.DataPetriNet;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.util.ui.widgets.ProMScrollPane;
import org.processmining.framework.util.ui.widgets.ProMTable;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.PNWDTransition;
import org.processmining.plugins.utils.ProvidedObjectHelper;
import org.processmining.xesalignmentextension.XAlignmentExtension;
import org.processmining.xesalignmentextension.XAlignmentExtension.XAlignmentMove;
import org.processmining.xesalignmentextension.XDataAlignmentExtension;
import org.processmining.xesalignmentextension.XDataAlignmentExtension.IncorrectXAttribute;
import org.processmining.xesalignmentextension.XDataAlignmentExtension.XDataAlignmentExtensionException;
import org.processmining.xesalignmentextension.XDataAlignmentExtension.XDataAlignmentMove;

import com.google.common.base.Predicate;
import com.google.common.collect.Iterables;
import com.google.common.collect.Ordering;

public class AttributesComparisonPanel extends JPanel {

	private final static class FilterAlignmentAttribute implements Predicate<XAttribute> {
		public boolean apply(XAttribute attribute) {
			if (attribute.getExtension() == XAlignmentExtension.instance()) {
				return false;
			}
			String key = attribute.getKey();
			return !key.startsWith("alignment:") && !key.startsWith("dataalignment:");
		}
	}

	private static final DateFormat DATE_FORMATTER = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.FULL,
			Locale.ENGLISH);

	private static final Ordering<XAttribute> DEFAULT_ATTRIBUTE_ORDERING = new Ordering<XAttribute>() {

		public int compare(XAttribute arg0, XAttribute arg1) {
			return arg0.getKey().compareTo(arg1.getKey());
		}
	};

	private static final Ordering<IncorrectXAttribute> DEFAULT_INCORRECT_ATTRIBUTE_ORDERING = new Ordering<IncorrectXAttribute>() {

		public int compare(IncorrectXAttribute arg0, IncorrectXAttribute arg1) {
			return arg0.getModelAttribute().getKey().compareTo(arg1.getModelAttribute().getKey());
		}
	};

	private static final long serialVersionUID = 1L;

	private final JTable variableTable;
	private final DefaultTableModel tableModel;

	private final PluginContext context;

	public AttributesComparisonPanel() {
		this(null);
	}

	@SuppressWarnings("serial")
	public AttributesComparisonPanel(PluginContext context) {
		super();
		this.context = context;
		setBackground(null);
		setForeground(null);
		setOpaque(false);
		setLayout(new BorderLayout());
		setMinimumSize(new Dimension(100, 150));

		tableModel = new DefaultTableModel(new String[] { "Name", "Process Value", "Log Value" }, 0);
		ProMTable promTable = new ProMTable(tableModel);
		variableTable = promTable.getTable();
		variableTable.setMaximumSize(null);
		variableTable.setPreferredSize(null);
		variableTable.setBackground(null);
		variableTable.setForeground(null);
		variableTable.setOpaque(false);
		variableTable.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
		DefaultTableCellRenderer cellRenderer = new DefaultTableCellRenderer() {

			public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected,
					boolean hasFocus, int row, int column) {
				Component c = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
				c.setBackground(AttributesComparisonPanel.this.getParent().getBackground());
				c.setForeground(AttributesComparisonPanel.this.getParent().getForeground());
				return c;
			}

		};
		Enumeration<TableColumn> columns = variableTable.getColumnModel().getColumns();
		while (columns.hasMoreElements()) {
			final TableColumn column = columns.nextElement();
			column.setPreferredWidth(130);
			column.setCellRenderer(cellRenderer);
			column.setCellEditor(new DefaultCellEditor(new JTextField()) {

				protected void fireEditingStopped() {
					this.cancelCellEditing();
					super.fireEditingStopped();
				}

				protected void fireEditingCanceled() {
					super.fireEditingCanceled();
				}

			});
		}
		variableTable.getColumnModel().setColumnSelectionAllowed(false);
		variableTable.getTableHeader().setReorderingAllowed(false);
		variableTable.getColumnModel().getColumn(2).setCellRenderer(new DefaultTableCellRenderer() {

			public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected,
					boolean hasFocus, int row, int column) {
				Component cell = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
				cell.setForeground(Color.BLACK);
				cell.setBackground(null);
				TableModel model = table.getModel();
				if (column == 1) {
					Object otherValue = model.getValueAt(row, column + 1);
					if (!isNotAvailable(value, otherValue) && !otherValue.equals(value)) {
						colorCell(cell, Color.RED);
					}
				} else if (column == 2) {
					Object otherValue = model.getValueAt(row, column - 1);
					if (!isNotAvailable(value, otherValue) && !model.getValueAt(row, column - 1).equals(value)) {
						colorCell(cell, Color.RED);
					}
				}
				return cell;
			}

			private boolean isNotAvailable(Object value, Object otherValue) {
				return "-".equals(otherValue) || "-".equals(value);
			}

			private void colorCell(Component cell, Color color) {
				Color bg = cell.getBackground();
				cell.setForeground(color);
				cell.setBackground(bg);
			}

		});

		promTable.removeAll();

		ProMScrollPane scrollPane = new ProMScrollPane(variableTable);

		scrollPane.setBackground(null);
		scrollPane.setForeground(null);
		scrollPane.setOpaque(false);
		scrollPane.getViewport().setBackground(null);
		scrollPane.getViewport().setForeground(null);
		scrollPane.getViewport().setOpaque(false);

		add(scrollPane, BorderLayout.CENTER);
	}

	public void displayAlignmentMove(XAlignmentMove move) throws XDataAlignmentExtensionException {
		tableModel.getDataVector().removeAllElements();

		XDataAlignmentMove dataAlignmentMove = XDataAlignmentExtension.instance().extendXAlignmentMove(move);

		switch (move.getType()) {
			case SYNCHRONOUS :
				addGuard(dataAlignmentMove);
				addSeparator("Standard Attributes");
				addCommonAttributes(dataAlignmentMove);
				addSeparator("Variables in DPN");
				addModelAttributes(dataAlignmentMove.getCorrectAttributes(), dataAlignmentMove.getIncorrectAttributes(),
						dataAlignmentMove.getMissingAttributes());
				addSeparator("Other Log Attributes");
				addOtherLogAttributes(dataAlignmentMove.getUnmappedAttributes());
				break;

			case LOG :
				addSeparator("Standard Attributes");
				addCommonAttributes(dataAlignmentMove);
				addSeparator("Remaining Log Attributes");
				addOtherLogAttributes(dataAlignmentMove.getUnmappedAttributes());
				break;

			case MODEL :
				addGuard(dataAlignmentMove);
				addSeparator("Variables in DPN");
				addModelAttributes(dataAlignmentMove.getCorrectAttributes(), dataAlignmentMove.getIncorrectAttributes(),
						dataAlignmentMove.getMissingAttributes());
				break;
		}
		variableTable.repaint();
	}

	private void addGuard(XDataAlignmentMove move) {
		if (context != null) {
			String activityId = move.getActivityId();
			for (DataPetriNet net : ProvidedObjectHelper.getProvidedObjects(context, DataPetriNet.class)) {
				for (Transition t : net.getTransitions()) {
					if (activityId.equals(t.getLocalID().toString())) {
						if (t instanceof PNWDTransition && ((PNWDTransition) t).getGuardExpression() != null) {
							addSeparator("Transition Guard");
							tableModel.addRow(new String[] { "Guard",
									((PNWDTransition) t).getGuardExpression().toString(), "-" });
						}

					}
				}
			}
		}
	}

	private void addModelAttributes(List<XAttribute> correctAttributes, List<IncorrectXAttribute> incorrectAttributes,
			List<XAttribute> missingAttributes) {

		Iterator<IncorrectXAttribute> incorrectIter = DEFAULT_INCORRECT_ATTRIBUTE_ORDERING
				.immutableSortedCopy(incorrectAttributes).iterator();
		if (incorrectIter.hasNext()) {
			while (incorrectIter.hasNext()) {
				IncorrectXAttribute incorrectVar = incorrectIter.next();
				tableModel.addRow(new String[] { incorrectVar.getModelAttribute().getKey(),
						nullSafeFormat(getAttributeValue(incorrectVar.getModelAttribute()), "MISSING"),
						nullSafeFormat(getAttributeValue(incorrectVar.getLogAttribute()), "MISSING") });
			}
		}

		Iterator<XAttribute> missingIter = DEFAULT_ATTRIBUTE_ORDERING.immutableSortedCopy(missingAttributes).iterator();
		if (missingIter.hasNext()) {
			while (missingIter.hasNext()) {
				XAttribute processVar = missingIter.next();
				tableModel.addRow(new String[] { processVar.getKey(),
						nullSafeFormat(getAttributeValue(processVar), "MISSING"), "N/A" });
			}
		}

		Iterator<XAttribute> correctIter = DEFAULT_ATTRIBUTE_ORDERING.immutableSortedCopy(correctAttributes).iterator();
		if (correctIter.hasNext()) {
			while (correctIter.hasNext()) {
				XAttribute correctVar = correctIter.next();
				tableModel.addRow(
						new String[] { correctVar.getKey(), nullSafeFormat(getAttributeValue(correctVar), "MISSING"),
								nullSafeFormat(getAttributeValue(correctVar), "MISSING") });
			}
		}

	}

	public void displayTrace(String traceName, double fitness, XTrace trace) {
		tableModel.getDataVector().removeAllElements();

		addSeparator("Standard Attributes");

		tableModel.addRow(new String[] { "Name", "-", traceName });
		tableModel.addRow(new String[] { "Fitness", "-", NumberFormat.getNumberInstance().format(fitness) });

		addSeparator("Trace Attributes");

		if (trace != null) {

			for (XAttribute attr : trace.getAttributes().values()) {
				tableModel.addRow(new String[] { attr.getKey(), "-", attr.toString() });
			}
		}

		variableTable.repaint();
	}

	private void addSeparator(String name) {
		tableModel.addRow(
				new String[] { "-- " + name + "--", "----------------------------", "----------------------------" });
	}

	private void addOtherLogAttributes(List<XAttribute> otherAttributes) {
		Iterator<XAttribute> iter = DEFAULT_ATTRIBUTE_ORDERING
				.immutableSortedCopy(Iterables.filter(otherAttributes, new FilterAlignmentAttribute())).iterator();
		while (iter.hasNext()) {
			XAttribute attribute = iter.next();
			if (attribute.getExtension() != XConceptExtension.instance()
					&& attribute.getExtension() != XTimeExtension.instance()) {
				tableModel.addRow(new String[] { attribute.getKey(), "-", attribute.toString() });
			}
		}
	}

	private Object getAttributeValue(XAttribute attribute) {
		if (attribute instanceof XAttributeList) {
			return ((XAttributeList) attribute).getCollection();
		} else if (attribute instanceof XAttributeContainer) {
			return ((XAttributeContainer) attribute).getCollection();
		} else if (attribute instanceof XAttributeLiteral) {
			return ((XAttributeLiteral) attribute).getValue();
		} else if (attribute instanceof XAttributeBoolean) {
			return ((XAttributeBoolean) attribute).getValue();
		} else if (attribute instanceof XAttributeContinuous) {
			return ((XAttributeContinuous) attribute).getValue();
		} else if (attribute instanceof XAttributeDiscrete) {
			return ((XAttributeDiscrete) attribute).getValue();
		} else if (attribute instanceof XAttributeTimestamp) {
			return ((XAttributeTimestamp) attribute).getValue();
		} else if (attribute instanceof XAttributeID) {
			return ((XAttributeID) attribute).getValue();
		} else {
			throw new IllegalArgumentException("Unexpected attribute type!");
		}
	}

	private void addCommonAttributes(XAlignmentMove move) {
		// Transition / Name mapping
		String processStepName = move.getModelMove();
		String logStepName = move.getLogMove();
		tableModel.addRow(
				new String[] { "Label", nullSafeFormat(processStepName, "-"), nullSafeFormat(logStepName, "-") });

		// Time
		Date time = XTimeExtension.instance().extractTimestamp(move.getEvent());
		if (time != null) {
			tableModel.addRow(new String[] { "Timestamp", "-", nullSafeFormat(time, "--") });
		}
	}

	private static String nullSafeFormat(Object obj, String nullLabel) {
		if (obj == null) {
			return nullLabel;
		} else {
			if (obj instanceof Date) {
				return DATE_FORMATTER.format(obj);
			} else {
				return obj.toString();
			}
		}
	}

	public JTable getVariableTable() {
		return variableTable;
	}

}
