package org.processmining.plugins.DataConformance.visualization.alignment;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.deckfour.xes.model.XAttribute;
import org.processmining.framework.util.ui.widgets.traceview.ProMTraceView.AbstractExtendedEvent;
import org.processmining.xesalignmentextension.XAlignmentExtension.XAlignmentMove;
import org.processmining.xesalignmentextension.XDataAlignmentExtension;
import org.processmining.xesalignmentextension.XDataAlignmentExtension.IncorrectXAttribute;
import org.processmining.xesalignmentextension.XDataAlignmentExtension.XDataAlignmentExtensionException;
import org.processmining.xesalignmentextension.XDataAlignmentExtension.XDataAlignmentMove;

public class XAlignmentEvent extends AbstractExtendedEvent {

	private static final String NOMOVE = ">>";

	private final XDataAlignmentMove move;
	
	private boolean colorCodeActivities;
	private final Map<String, Color> activityColorMap;

	private List<String> attributes;
	
	public XAlignmentEvent(XAlignmentMove move, Map<String, Color> activityColorMap, boolean colorCodeActivities) {
		this.activityColorMap = activityColorMap;
		this.colorCodeActivities = colorCodeActivities;
		this.move = XDataAlignmentExtension.instance().extendXAlignmentMove(move);
	}

	public String getTopLabel() {
		return null;
	}

	public String getBottomLabel() {
		return getAttributes().size() >= 1 ? getAttributes().get(0) : null;
	}

	public String getBottomLabel2() {
		return getAttributes().size() == 2 ? getAttributes().get(1) : (getAttributes().size() > 2 ? getAttributes()
				.get(1).concat("..") : null);
	}

	public String getLabel() {
		if (move.getModelMove() != null) {
			return move.getModelMove();
		} else if (move.getLogMove() != null) {
			return move.getLogMove();
		} else {
			return NOMOVE;
		}
	}

	public Color getWedgeColor() {
		if (colorCodeActivities) {
			if (!move.isObservable()) {
				return Color.LIGHT_GRAY;
			} else if (move.getActivityId() != null) {
				Color color = activityColorMap.get(move.getActivityId());
				if (color != null) {
					return color;	
				} else {
					return Color.LIGHT_GRAY;
				}										
			} else {
				return Color.LIGHT_GRAY;
			}
		} else {
			return ColorTheme.getMoveColor(move);
		}
	}

	public Color getExtendedColor() {
		if (colorCodeActivities) {
			return ColorTheme.getMoveColor(move);
		} else {
			return null;	
		}		
	}

	private List<String> getAttributes() {
		if (attributes == null) {
			try {
				attributes = buildVariableAssignmentList(move);
			} catch (XDataAlignmentExtensionException e) {
				e.printStackTrace();
				attributes = Collections.emptyList();
			}
		}
		return attributes;
	}

	private static void sortAttributesByName(final List<XAttribute> attributes) {
		Collections.sort(attributes, new Comparator<XAttribute>() {

			public int compare(XAttribute o1, XAttribute o2) {
				return o1.getKey().compareTo(o2.getKey());
			}

		});
	}
	
	private static void sortIncorrectAttributesByName(final List<IncorrectXAttribute> attributes) {
		Collections.sort(attributes, new Comparator<IncorrectXAttribute>() {

			public int compare(IncorrectXAttribute o1, IncorrectXAttribute o2) {
				return o1.getModelAttribute().getKey().compareTo(o2.getModelAttribute().getKey());
			}

		});
	}

	private static List<String> buildVariableAssignmentList(XDataAlignmentMove move) throws XDataAlignmentExtensionException {
		
		List<String> attributes;

		List<IncorrectXAttribute> incorrectAttributes = move.getIncorrectAttributes();
		sortIncorrectAttributesByName(incorrectAttributes);
		
		List<XAttribute> missingAttributes = move.getMissingAttributes();
		sortAttributesByName(missingAttributes);

		List<XAttribute> correctAttributes = move.getCorrectAttributes();
		sortAttributesByName(correctAttributes);
		
		attributes = new ArrayList<>(incorrectAttributes.size() + missingAttributes.size() + correctAttributes.size());
		
		for (Iterator<IncorrectXAttribute> iterator = incorrectAttributes.iterator(); iterator
				.hasNext();) {
			IncorrectXAttribute incorrectEntry = iterator.next();
			XAttribute processAttr = incorrectEntry.getModelAttribute();
			XAttribute logAttr = incorrectEntry.getLogAttribute();

			Object correctValue = processAttr.toString();
			Object wrongValue = logAttr.toString();
	
			attributes.add(processAttr.getKey() + ":=" + correctValue + '\u2260' + wrongValue);
		}

		for (Iterator<XAttribute> iterator = missingAttributes.iterator(); iterator
				.hasNext();) {
			XAttribute a = iterator.next();				
			attributes.add(a.getKey() + ":=" + a.toString() + '\u2260' + "N/A");
		}
		
		for (Iterator<XAttribute> iterator = correctAttributes.iterator(); iterator
				.hasNext();) {
			XAttribute a = iterator.next();				
			Object processValue = a.toString();
			attributes.add(a.getKey() + ":=" + processValue);
		}

		return attributes;
	}

}
