package org.processmining.plugins.DataConformance.visualization.grouping;

import java.awt.Component;
import java.awt.Font;
import java.text.NumberFormat;
import java.util.Collection;
import java.util.Comparator;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.table.DefaultTableModel;

import org.apache.commons.math3.stat.StatUtils;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.util.ui.widgets.ProMTableWithoutPanel;
import org.processmining.framework.util.ui.widgets.traceview.ProMTraceList;
import org.processmining.framework.util.ui.widgets.traceview.masterdetail.DetailView;
import org.processmining.plugins.DataConformance.visualization.alignment.AlignmentLegend;
import org.processmining.plugins.DataConformance.visualization.alignment.AlignmentView;
import org.processmining.plugins.DataConformance.visualization.alignment.AlignmentView.Layout;
import org.processmining.plugins.DataConformance.visualization.grouping.GroupedAlignmentMasterView.GroupedAlignmentInput;
import org.processmining.xesalignmentextension.XAlignmentExtension.XAlignment;

public class GroupedAlignmentDetailView extends JPanel implements DetailView<XAlignment> {

	public interface SearchAction {
		void search(String query);
	}

	private static final long serialVersionUID = 1L;

	private final AlignmentView alignmentView;
	private final DefaultTableModel statisticsModel;

	public GroupedAlignmentDetailView(PluginContext context, GroupedAlignmentInput<XAlignment> input) {

		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		setOpaque(false);
		setBackground(null);
		setForeground(null);
		
		this.alignmentView = new AlignmentView(Layout.ONECOLUMN, context, input.traceResolver, input.activityColorMap);
		add(new AlignmentLegend());

		add(createHeading("ALIGNMENT STATISTICS"));
		add(Box.createVerticalStrut(5));

		statisticsModel = new DefaultTableModel();
		statisticsModel.addColumn("Name");
		statisticsModel.addColumn("Value");
		add(new ProMTableWithoutPanel(statisticsModel));

		add(Box.createVerticalStrut(10));
		
		add(createHeading("ALIGNMENTS IN SELECTED GROUPS"));
		add(alignmentView);

	}

	public AlignmentView getAlignmentView() {
		return alignmentView;
	}

	public JComponent getDetailComponent() {
		return this;
	}

	public ProMTraceList<XAlignment> getDetailList() {
		return alignmentView.getDetailList();
	}

	public Comparator<XAlignment> getSortOrder() {
		return alignmentView.getSortOrder();
	}

	public void updateStatistics(Collection<XAlignment> alignments) {
		double[] fitness = new double[alignments.size()];
		int i = 0;
		for (XAlignment a : alignments) {
			fitness[i++] = a.getFitness();
		}
		statisticsModel.getDataVector().clear();

		NumberFormat format = NumberFormat.getPercentInstance();
		statisticsModel.addRow(new String[] { "#Traces", NumberFormat.getIntegerInstance().format(alignments.size()) });
		statisticsModel.addRow(new String[] { "Min Fitness", format.format(StatUtils.min(fitness)) });
		statisticsModel.addRow(new String[] { "Average Fitness", format.format(StatUtils.mean(fitness)) });
		statisticsModel.addRow(new String[] { "Median Fitness", format.format(StatUtils.percentile(fitness, 50)) });
		statisticsModel.addRow(new String[] { "Max Fitness", format.format(StatUtils.max(fitness)) });
		statisticsModel.fireTableDataChanged();
	}

	private JLabel createHeading(String label) {
		JLabel title = new JLabel(label);
		title.setAlignmentX(Component.CENTER_ALIGNMENT);
		title.setFont(title.getFont().deriveFont(Font.BOLD));
		title.setForeground(null);
		return title;
	}

}