package org.processmining.plugins.DataConformance.visualization.grouping;

import java.awt.Color;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JComponent;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.plugins.DataConformance.Alignment;
import org.processmining.plugins.DataConformance.ResultReplay;
import org.processmining.plugins.DataConformance.visualization.alignment.ColorTheme;
import org.processmining.plugins.DataConformance.visualization.alignment.XTraceResolver;
import org.processmining.plugins.DataConformance.visualization.grouping.GroupedAlignmentMasterView.GroupedAlignmentInput;
import org.processmining.plugins.DataConformance.visualization.grouping.GroupedAlignmentsSimpleImpl.GroupingFunction;
import org.processmining.plugins.balancedconformance.export.XAlignmentConverter;
import org.processmining.plugins.balancedconformance.result.AlignmentCollection;
import org.processmining.xesalignmentextension.XAlignmentExtension.XAlignment;

import com.google.common.base.Function;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.Iterables;

/**
 * Visualize a group of similar alignments based on a {@link GroupingFunction}
 * 
 * @author F. Mannhardt
 * 
 */
@Plugin(name = "3 Alignments Grouped by Variant", level = PluginLevel.PeerReviewed, //
returnLabels = { "Model-Log Alignment in Presence of Data" }, returnTypes = { JComponent.class },//
parameterLabels = { "Matching Instances" }, userAccessible = true)
@Visualizer
public class GroupedAlignmentVisualizer {

	@PluginVariant(requiredParameterLabels = { 0 })
	public JComponent visualise(PluginContext context, AlignmentCollection alignments) {
		XAlignmentConverter converter = new XAlignmentConverter();
		if (alignments instanceof ResultReplay) {
			ResultReplay resultReplay = (ResultReplay) alignments;
			XTraceResolver traceResolver = buildTraceMap(resultReplay);
			
			converter.setClassifier(resultReplay.getClassifier());
			converter.setVariableMapping(resultReplay.getVariableMapping());
			
			Iterable<XAlignment> xAlignments = convertToXAlignment(alignments, converter, traceResolver);
			Map<String, Color> activityColorMap = ColorTheme.createColorMap(xAlignments);
			return new GroupedAlignmentMasterDetail(context, new GroupedAlignmentInput<>(
					new GroupedAlignmentsSimpleImpl(xAlignments, activityColorMap), traceResolver, activityColorMap));
		} else {
			// No information about original log
			Iterable<XAlignment> xAlignments = convertToXAlignment(alignments, converter);
			Map<String, Color> activityColorMap = ColorTheme.createColorMap(xAlignments);
			return new GroupedAlignmentMasterDetail(context, new GroupedAlignmentInput<>(
					new GroupedAlignmentsSimpleImpl(xAlignments, activityColorMap), new XTraceResolver() {

						public boolean hasOriginalTraces() {
							return false;
						}

						public XTrace getOriginalTrace(String name) {
							return null;
						}
					}, activityColorMap));
		}
	}

	private XTraceResolver buildTraceMap(ResultReplay logReplayResult) {
		final Map<String, XTrace> traceMap = new HashMap<>();
		for (XTrace trace : logReplayResult.getAlignedLog()) {
			traceMap.put(XConceptExtension.instance().extractName(trace), trace);
		}
		return new XTraceResolver() {

			public boolean hasOriginalTraces() {
				return true;
			}

			public XTrace getOriginalTrace(String name) {
				return traceMap.get(name);
			}
		};
	}

	private Iterable<XAlignment> convertToXAlignment(AlignmentCollection alignments, final XAlignmentConverter converter,
			final XTraceResolver resolver) {
		return ImmutableList.copyOf(Iterables.transform(alignments.getAlignments(), new Function<Alignment, XAlignment>() {

			public XAlignment apply(Alignment a) {
				return converter.viewAsXAlignment(a, resolver.getOriginalTrace(a.getTraceName()));
			}
		}));
	}

	private Iterable<XAlignment> convertToXAlignment(AlignmentCollection alignments, final XAlignmentConverter converter) {
		return ImmutableList.copyOf(Iterables.transform(alignments.getAlignments(), new Function<Alignment, XAlignment>() {

			public XAlignment apply(Alignment a) {
				return converter.viewAsXAlignment(a);
			}
		}));
	}

}
