package org.processmining.plugins.DataConformance.visualization.grouping;

import java.awt.Color;
import java.util.Map;

import javax.swing.JComponent;

import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.plugins.DataConformance.visualization.alignment.ColorTheme;
import org.processmining.plugins.DataConformance.visualization.alignment.XTraceResolver;
import org.processmining.plugins.DataConformance.visualization.grouping.GroupedAlignmentMasterView.GroupedAlignmentInput;
import org.processmining.plugins.DataConformance.visualization.grouping.GroupedAlignmentsSimpleImpl.GroupingFunction;
import org.processmining.xesalignmentextension.XAlignmentExtension;
import org.processmining.xesalignmentextension.XAlignmentExtension.XAlignment;

import com.fluxicon.slickerbox.factory.SlickerFactory;
import com.google.common.base.Function;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.Iterables;

/**
 * Visualize a group of similar alignments based on a {@link GroupingFunction}
 * 
 * @author F. Mannhardt
 * 
 */
@Plugin(name = "Log with AlignmentExtension (Grouped)", level = PluginLevel.Regular,//
returnLabels = { "Model-Log Alignment in Presence of Data" }, returnTypes = { JComponent.class },//
parameterLabels = { "Matching Instances" }, userAccessible = true)
@Visualizer
public class GroupedXAlignmentExtensionVisualizer {

	@PluginVariant(requiredParameterLabels = { 0 })
	public JComponent visualise(PluginContext context, XLog extendedLog) {

		if (hasAlignmentExtension(extendedLog)) {
			Iterable<XAlignment> alignments = ImmutableList.copyOf(Iterables.transform(extendedLog, new Function<XTrace, XAlignment>() {

				public XAlignment apply(XTrace trace) {
					return XAlignmentExtension.instance().extendTrace(trace);
				}
			}));
			Map<String, Color> activityColorMap = ColorTheme.createColorMap(alignments);
			return new GroupedAlignmentMasterDetail(context, new GroupedAlignmentInput<>(
					new GroupedAlignmentsSimpleImpl(alignments, activityColorMap), new XTraceResolver() {

						public boolean hasOriginalTraces() {
							return false;
						}

						public XTrace getOriginalTrace(String name) {
							return null;
						}
					}, activityColorMap));
		} else {
			return SlickerFactory.instance().createLabel("XAlignmentExtension is missing!");
		}
	}

	private static boolean hasAlignmentExtension(XLog extendedLog) {
		boolean hasExtensionOnLogLevel = extendedLog.getExtensions().contains(XAlignmentExtension.instance());
		// As the extension is not correctly parsed when the XAlignmentExtension class was not yet loaded 
		boolean hasAlignmentAttribute = XAlignmentExtension.instance().extractFitness(extendedLog) != null;
		return hasExtensionOnLogLevel || hasAlignmentAttribute;
	}

}
