package org.processmining.plugins.balancedconformance.controlflow.override.queues.noduplicate;

import java.util.Random;

import org.processmining.plugins.astar.petrinet.impl.AbstractPDelegate;
import org.processmining.plugins.balancedconformance.controlflow.override.DRecord;
import org.processmining.plugins.balancedconformance.controlflow.override.queues.RandomizedQueue;

import it.unimi.dsi.fastutil.Hash.Strategy;
import nl.tue.astar.Tail;

/**
 * Random, consistent secondary order in the priority queue
 * 
 * @author F. Mannhardt
 *
 */
public final class RandomDataPriorityQueue extends DataFastLookupPriorityQueue implements RandomizedQueue {

	private int randomSeed = new Random().nextInt(Integer.MAX_VALUE);

	public RandomDataPriorityQueue(int initialCapacity, Strategy<DRecord> strategy,
			AbstractPDelegate<? extends Tail> delegate, int maxCost) {
		super(initialCapacity, strategy, delegate, maxCost);
	}

	public RandomDataPriorityQueue(int initialCapacity, Strategy<DRecord> strategy,
			AbstractPDelegate<? extends Tail> delegate) {
		super(initialCapacity, strategy, delegate);
	}

	@Override
	public boolean isBetter(DRecord r1, DRecord r2) {
		// retrieve stored cost 
		double c1 = r1.getTotalCost();
		double c2 = r2.getTotalCost();

		if (Double.compare(c1, c2) == 0) {
			int p1 = pseudoRandomInt(r1);
			int p2 = pseudoRandomInt(r2);
			int d1 = Math.abs(p1 - randomSeed);
			int d2 = Math.abs(p2 - randomSeed);
			return d1 < d2;
		}
		return Double.compare(c1, c2) < 0;
	}

	private int pseudoRandomInt(DRecord record) {
		return System.identityHashCode(record);
	}

}