/*
 * Copyright (c) 2013 F. Mannhardt (f.mannhardt@tue.nl)
 */
package org.processmining.plugins.balancedconformance.controlflow.resultbuilder;

import org.processmining.plugins.balancedconformance.controlflow.ControlFlowAlignmentException;
import org.processmining.plugins.balancedconformance.controlflow.ControlFlowAlignmentResult;
import org.processmining.plugins.balancedconformance.controlflow.UnreliableControlFlowAlignmentException;

/**
 * Extends {@link ControlFlowAlignmentResultBuilder} with information about data
 * alignments.
 * 
 * @author F. Mannhardt
 * 
 */
public interface AlignmentResultBuilder {

	/**
	 * Returns the {@link ControlFlowAlignmentResult}
	 * 
	 * @param upperBound
	 *            above this cost, alignments do not need to be considered
	 * @return the next alignment
	 * @throws ControlFlowAlignmentException
	 *             in case something really goes wrong
	 * @throws UnreliableControlFlowAlignmentException
	 *             in case the returned alignment is not reliable because some
	 *             limit was reached
	 */
	ControlFlowAlignmentResult calcResult(double upperBound) throws ControlFlowAlignmentException;

	/**
	 * @return how much states in the alignment search space are queued
	 */
	long getTotalQueuedStates();

	/**
	 * @return how many partial data alignments (data alignment for a alignment
	 *         prefix) have been done
	 */
	int getPartialDataAlignments();

	/**
	 * @return the size of the cache for partial data alignments
	 */
	int getCacheSize();

	/**
	 * @return how often was a cached partial data alignment used
	 */
	int getCacheHit();

	/**
	 * @return how many data states are in the current state space
	 */
	long getDataStateCount();

}
