/*
 * Copyright (c) 2013 F. Mannhardt (f.mannhardt@tue.nl)
 */
package org.processmining.plugins.balancedconformance.dataflow;

import org.processmining.plugins.balancedconformance.dataflow.datastate.DataState;
import org.processmining.plugins.balancedconformance.dataflow.datastate.DataStateFactory;
import org.processmining.plugins.balancedconformance.mapping.LogMapping;

/**
 * Base class for most of the more expensive {@link DataAlignmentAdapter}
 * 
 * @author F. Mannhardt
 *
 */
public abstract class AbstractDataAlignmentAdapter implements DataAlignmentAdapter {

	protected final LogMapping logMapping;
	protected final DataStateFactory dataStateFactory;
	
	public AbstractDataAlignmentAdapter(LogMapping mapping, DataStateFactory dataStateFactory) {
		super();
		this.logMapping = mapping;
		this.dataStateFactory = dataStateFactory;
	}
	
	/**
	 * @return an empty mutable DataState
	 */
	protected DataState newDataState() {
		return dataStateFactory.newDataState();
	}

	public DataStateFactory getDataStateFactory() {
		return dataStateFactory;
	}
	
	protected int convertDataCost(float dataCost, double delta) {
		int newDataCost;
		if (dataCost == Float.MAX_VALUE) {
			newDataCost = (int) Math.ceil(Integer.MAX_VALUE);
		} else {
			newDataCost = (int) Math.ceil(dataCost * delta);
		}
		return newDataCost;
	}
	
	
}
