package org.processmining.plugins.balancedconformance.dataflow;

import org.processmining.plugins.DataConformance.framework.ExecutionTrace;
import org.processmining.plugins.balancedconformance.controlflow.ControlFlowAlignmentResult;
import org.processmining.plugins.balancedconformance.dataflow.exception.MILPProblemBuildException;
import org.processmining.plugins.balancedconformance.dataflow.exception.MILPProblemException;
import org.processmining.plugins.balancedconformance.mapping.TraceMapping;

/**
 * 
 * 
 * @author F. Mannhardt
 *
 */
public interface MILPProblemBuilder {

	/*
	 * Timeout in seconds for solving one ILP problem
	 */
	public static final int TIMEOUT = 360;

	/*
	 * Difference between
	 */
	public static final double RESULT_EPSILON = 0.01d;

	/*
	 * Precision of the ILP solver
	 */
	public static final double SOLVING_PRECISION = 0.000000001d;

	public interface FullDataAlignedTrace {

		public final static FullDataAlignedTrace IMPOSSIBLE = new FullDataAlignedTrace() {

			public ExecutionTrace getProcessTracePrefix() {
				return null;
			}

			public ExecutionTrace getLogTracePrefix() {
				return null;
			}

			public int getDataCost() {
				return Integer.MAX_VALUE;
			}
		};

		int getDataCost();

		ExecutionTrace getProcessTracePrefix();

		ExecutionTrace getLogTracePrefix();

	}

	/**
	 * @param controlFlowAlignment
	 * @return
	 * @throws MILPProblemBuildException
	 */
	FullDataAlignedTrace createAlignment(TraceMapping mapping, ControlFlowAlignmentResult controlFlowAlignment)
			throws MILPProblemException;

	/**
	 * @param record
	 * @param controlFlowAlignment
	 * @return
	 * @throws MILPProblemBuildException
	 */
	PartialDataAlignedTrace createPartialAlignment(TraceMapping mapping, ControlFlowAlignmentResult controlFlowAlignment)
			throws MILPProblemException;

}
