/*
 * Copyright (c) 2013 F. Mannhardt (f.mannhardt@tue.nl)
 */
package org.processmining.plugins.balancedconformance.dataflow.datastate;

/**
 * Stores the values of a (typically small) number of variables that are of the
 * types Long, Double or String.
 * 
 * @author F. Mannhardt
 * 
 */
public interface DataState {

	public interface VariableEntryProcedure {
		boolean execute(String varName, Object value);
	}

	void putDouble(String varName, double value);

	void putLong(String varName, long value);

	long getLong(String varName);
	
	double getDouble(String varName);

	Double tryGetDouble(String varName);

	Long tryGetLong(String varName);

	Long tryGetPrimeOrNormalLong(String varName);

	Double tryGetPrimeOrNormalDouble(String varName);

	boolean contains(String varName);

	boolean isEmpty();

	int size();

	/**
	 * Turn all variables with a prime (') symbol into normal variables
	 * overwriting existing values
	 */
	void movePrimes();

	/**
	 * @return an immutable version of this state (might return a clone or the
	 *         original)
	 */
	DataState makeStateImmutable();

	/**
	 * 
	 * @param movePrimes
	 *            move primes while cloning
	 * @return a deep clone of this instance
	 */
	DataState clone(boolean movePrimes);

	/**
	 * @return a reference to the underlying data structure
	 */
	Object getData();

}
