/*
 * Copyright (c) 2013 F. Mannhardt (f.mannhardt@tue.nl)
 */
package org.processmining.plugins.balancedconformance.dataflow.datastate;

/**
 * Immutable Data State that represents the empty state
 * 
 * @author F. Mannhardt
 * 
 */
public class EmptyDataState implements DataState {

	private long[] emptyData = new long[] {};

	public void putDouble(String varName, double value) {
		throw new UnsupportedOperationException("EmptyDataState");
	}

	public void putLong(String varName, long value) {
		throw new UnsupportedOperationException("EmptyDataState");
	}

	public long getLong(String varName) {
		throw new RuntimeException("DataState does not contain variable " + varName + ": EmptyDataState");
	}

	public double getDouble(String varName) {
		throw new RuntimeException("DataState does not contain variable " + varName + ": EmptyDataState");
	}

	public boolean contains(String varName) {
		return false;
	}

	public boolean isEmpty() {
		return true;
	}

	public int size() {
		return 0;
	}

	public void forEachEntry(VariableEntryProcedure procedure) {
		throw new UnsupportedOperationException("EmptyDataState");
	}

	public void putAll(DataState state) {
		throw new UnsupportedOperationException("EmptyDataState");
	}

	public DataState clone(boolean movePrimes) {
		throw new UnsupportedOperationException("EmptyDataState");
	}

	public Double tryGetDouble(String varName) {
		return null;
	}

	public Long tryGetLong(String varName) {
		return null;
	}

	public Long tryGetPrimeOrNormalLong(String varName) {
		return null;
	}

	public Double tryGetPrimeOrNormalDouble(String varName) {
		return null;
	}

	public void movePrimes() {
	}

	public DataState makeStateImmutable() {
		return this;
	}

	public Object getData() {
		return emptyData;
	}

}
