package org.processmining.plugins.balancedconformance.dataflow.datastate.compression;

import java.util.Arrays;

/**
 * Protobuf VarInt, implementation adapted from <a href=
 * "https://github.com/addthis/stream-lib/blob/master/src/main/java/com/clearspring/analytics/util/Varint.java"
 * >stream-lib</a> (Apache 2.0 Licensed). Copied to avoid introducing a
 * dependency.
 */
public final class DeltaCompressor implements Compressor {

	public long[] decompress(byte[] compressedData, int uncompressedSize) {
		long[] uncompressed = new long[uncompressedSize];
		decompressLongs(compressedData, uncompressed);
		return uncompressed;
	}

	public static void decompressLongs(byte[] compressedData, long[] uncompressed) {
		int index = 0;
		long prev = 0l;
		for (int j = 0; j < uncompressed.length; j++) {
			long b = 0, val = 0;
			int i = 0;
			while (((b = compressedData[index++]) & 0x80L) != 0) {
				val |= (b & 0x7F) << i;
				i += 7;
			}
			val = val | (b << i);
			prev += zigzagDecode(val);
			uncompressed[j] = prev;
		}
	}

	public byte[] compress(long[] uncompressed) {
		byte[] buffer = new byte[uncompressed.length * 10];
		int index = compressLongs(uncompressed, buffer);
		return Arrays.copyOf(buffer, index);
	}

	public static int compressLongs(long[] uncompressed, byte[] buffer) {
		int index = 0;
		for (int i = 0; i < uncompressed.length; i++) {
			long prev = i == 0 ? 0 : uncompressed[i - 1];
			long delta = zigzagEncode(uncompressed[i] - prev);
			index = compressLong(buffer, index, delta);
		}
		return index;
	}

	private static int compressLong(byte[] buffer, int index, long val) {
		while ((val & 0xFFFFFFFFFFFFFF80L) != 0L) {
			buffer[index++] = (byte) (((int) val & 0x7F) | 0x80);
			val >>>= 7;
		}
		buffer[index++] = (byte) ((int) val & 0x7F);
		return index;
	}

	private static long zigzagDecode(long l) {
		return (l >>> 1) ^ (-(l & 1));
	}

	private static long zigzagEncode(long l) {
		return (l << 1) ^ (l >> 63);
	}

}