package org.processmining.plugins.balancedconformance.dataflow.util;

import java.util.Date;
import java.util.Map;

import org.processmining.datapetrinets.exception.VariableNotFoundException;
import org.processmining.datapetrinets.expression.VariableProvider;
import org.processmining.plugins.balancedconformance.dataflow.datastate.DataState;
import org.processmining.plugins.balancedconformance.dataflow.datastate.DataStateHelper;
import org.processmining.plugins.balancedconformance.mapping.StringDiscretizer;

public final class DataStateVariableProvider implements VariableProvider {

	private final DataState dataState;
	private final StringDiscretizer discretizer;
	@SuppressWarnings("rawtypes")
	private final Map<String, Class> variableTypes;

	public DataStateVariableProvider(DataState dataState, StringDiscretizer discretizer, @SuppressWarnings("rawtypes") Map<String, Class> variableTypes) {
		this.dataState = dataState;
		this.discretizer = discretizer;
		this.variableTypes = variableTypes;
	}

	public Object getValue(String variableName) throws VariableNotFoundException {
		Object result = null;
		Class<?> type = variableTypes.get(DataStateHelper.stripPrimeIfNeeded(variableName));
		if (type == null) {
			throw new VariableNotFoundException("Unkown variable  " + variableName);
		} else if (type == String.class) {
			Long tryValue = dataState.tryGetLong(variableName);
			if (tryValue != null) {
				assert tryValue < Integer.MAX_VALUE : "Invalid value for STRING variable " + variableName;
				result = DataStateHelper.unDiscretize(variableName, tryValue.intValue(), discretizer);
			}
		} else if (type == Long.class || type == Integer.class) {
			Long tryValue = dataState.tryGetLong(variableName);
			if (tryValue != null) {
				result = tryValue;
			}
		} else if (type == Boolean.class) {
			Long tryValue = dataState.tryGetLong(variableName);
			if (tryValue != null) {
				assert tryValue == 0 || tryValue == 1 : "Invalid Boolean in DataState " + dataState;
				result = tryValue == 0 ? Boolean.FALSE : Boolean.TRUE;
			}
		} else if (type == Date.class) {
			Long tryValue = dataState.tryGetLong(variableName);
			if (tryValue != null) {
				result = new Date(tryValue);
			}
		} else {
			Double tryValue = dataState.tryGetDouble(variableName);
			if (tryValue != null) {
				result = tryValue;
			}
		}
		return result;
	}

}