package org.processmining.plugins.balancedconformance.observer;

import java.util.Collection;
import java.util.logging.Level;

import org.deckfour.xes.model.XTrace;
import org.processmining.plugins.balancedconformance.result.BalancedDataAlignmentState;
import org.processmining.plugins.balancedconformance.result.DataAlignedTrace;

public class WrappedDataConformanceObserver implements DataConformancePlusObserver {

	private final DataConformancePlusObserver wrappedObserver;

	public WrappedDataConformanceObserver(DataConformancePlusObserver observer) {
		this.wrappedObserver = observer;
	}

	public void foundOptimalAlignment(int traceIndex, XTrace trace, final DataAlignedTrace alignedTrace,
			int partialDataAlignmentsNeeded, int cacheHit, int cacheSize, long queuedStates, long dataStateCount,
			long usedTime) {
		getWrappedObserver().foundOptimalAlignment(traceIndex, trace, alignedTrace, partialDataAlignmentsNeeded,
				cacheHit, cacheSize, queuedStates, dataStateCount, usedTime);
	}

	public void slowDataAlignmentDetected(int traceIndex, XTrace trace, DataAlignedTrace dataAlignmentState,
			long usedTime) {
		getWrappedObserver().slowDataAlignmentDetected(traceIndex, trace, dataAlignmentState, usedTime);
	}

	public void processingNewTrace(int traceIndex, XTrace trace) {
		getWrappedObserver().processingNewTrace(traceIndex, trace);
	}

	public void calculatedFitness(int traceIndex, XTrace trace, BalancedDataAlignmentState resultState) {
		getWrappedObserver().calculatedFitness(traceIndex, trace, resultState);
	}

	public void unreliableAlignmentDetected(int traceIndex, XTrace currentTrace) {
		getWrappedObserver().unreliableAlignmentDetected(traceIndex, currentTrace);
	}

	@SuppressWarnings("deprecation") // need for forward
	public void log(final String message) {
		getWrappedObserver().log(message);
	}

	@SuppressWarnings("deprecation") // need for forward
	public void log(final String message, final Throwable e) {
		getWrappedObserver().log(message, e);
	}

	public void log(Level level, String message) {
		getWrappedObserver().log(level, message);
	}

	public void finishedAlignment() {
		getWrappedObserver().finishedAlignment();
	}

	public void startAlignment(int numExpectedResults) {
		getWrappedObserver().startAlignment(numExpectedResults);
	}

	public DataConformancePlusObserver getWrappedObserver() {
		return wrappedObserver;
	}

	public void foundImpossibleAlignments(Collection<ImpossibleTrace> impossibleTraces) {
		getWrappedObserver().foundImpossibleAlignments(impossibleTraces);
	}

}
